
import sprite
import images
import random
import pygame
import math

class StartScreenAnimation(object):
    _HEIGHT = 310
    _LEFT_X = 280
    _RIGHT_X = 520
    
    def __init__(self):
        self.kid_animation = sprite.AnimatedSprite(images.IMG_KID, 8, 4)
        self.randomize()
        
    def randomize(self):
        kid_is_left = random.choice((True, False))
        animation_type = random.choice(range(4))
        self.kid_animation.reset_animation(animation_type, not kid_is_left)
        
        self.kid_rect = pygame.Rect(0, 0, 69, 71)
        self.jurlo_rect = pygame.Rect(0, 0, 50, 50)
        
        self.kid_rect.centery = self._HEIGHT
        self.jurlo_rect.centery = self._HEIGHT
        
        left_rect = self.kid_rect if kid_is_left else self.jurlo_rect
        right_rect = self.jurlo_rect if kid_is_left else self.kid_rect
        
        left_rect.centerx = self._LEFT_X
        right_rect.centerx = self._RIGHT_X
        
    def tick_and_draw(self, surface):
        self.kid_animation.next_frame()
        self.kid_animation.draw(surface, self.kid_rect)
        surface.blit(images.IMG_JURLO, self.jurlo_rect)
        

class Kid(object):
    _DIRECTION_RIGHT = False
    _DIRECTION_LEFT = True
    _ANIMATION_IDLE = 0
    _ANIMATION_RUN = 1
    _ANIMATION_FALL = 2
    _ANIMATION_HAPPY = 3
    
    _SPEED = 8
    
    def __init__(self):
        self.sprite = sprite.AnimatedSprite(images.IMG_KID, 8, 4)
        
        self.facing = Kid._DIRECTION_RIGHT
        
        self.hit_rect = pygame.Rect(0, 0, 50, 50)
        self.draw_rect = pygame.Rect(0, 0, 69, 71)
        self.hit_rect.bottom = 496
        self.hit_rect.centerx = 400
        self.draw_rect.bottom = 500
        self.draw_rect.centerx = 400
        
        self.good_hit_counter = 0
        self.bad_hit_counter = 0
        self.is_alive = True
        
    def tick(self, move_left, move_right):
        if self.bad_hit_counter > 0 or (not self.is_alive):
            self._tick_bad_hit()
        elif self.good_hit_counter > 0:
            self._tick_good_hit()
        else:
            self._tick_free(move_left, move_right)
       
    def _tick_free(self, move_left, move_right):
        if move_left and move_right:
            # if you can't decide where to go you won't move at all
            move_left = False
            move_right = False
        
        if move_left:
            self.facing = Kid._DIRECTION_LEFT
            animation = Kid._ANIMATION_RUN
            direction = -1
        elif move_right:
            self.facing = Kid._DIRECTION_RIGHT
            animation = Kid._ANIMATION_RUN
            direction = 1
        else:
            animation = Kid._ANIMATION_IDLE
            direction = 0
        # next animation frame:    
        self.sprite.next_frame(animation, self.facing)
        # move sprite on screen:
        x_off = direction * Kid._SPEED
        self.hit_rect.move_ip(x_off, 0)
        self.draw_rect.move_ip(x_off, 0)
    
    def _tick_good_hit(self):
        self.sprite.next_frame(Kid._ANIMATION_HAPPY, self.facing)
        self.good_hit_counter -= 1
    
    def _tick_bad_hit(self):
        self.sprite.next_frame(Kid._ANIMATION_FALL, self.facing)
        self.bad_hit_counter -= 1
        self.draw_rect.move_ip(0, 10) # fall down
        self.hit_rect.move_ip(0, 10)
        if (self.bad_hit_counter < 1): self.is_alive = False
        if (self.draw_rect.top >= 600): self.draw_rect.bottom = 0
        
        
    def draw(self, surface):
        self.sprite.draw(surface, self.draw_rect)
        # debug output:
        # pygame.draw.rect(surface, 0x00ff00, self.hit_rect, 1)
        
    def good_hit(self):
        self.good_hit_counter = 8
    
    def bad_hit(self):
        self.bad_hit_counter = 8
        

class ThingsEmitter(object):
    MAX_DEPTH = 500
    GOOD_HIT_INCREMENT = 10
    GOOD_MISSED_DECREMENT = 1
    
    def __init__(self, target):
        self.good_list = []
        self.bad_list = []
        self.explosion_list = []
        self.target = target
        self.random_things = ("good", "good", "bad",
                              "", "", "", "", "",
                              "", "", "", "", "",
                              "", "", "", "", "")
        self.random_velocities = range(9, 13)
        self.elapsed_ticks = 0
        self.score = 0
        self.bad_sprite_choices = range(3)
        self.good_sprite_choices = range(4)
        
        
    def tick(self):
        self.explosion_list = [explosion for explosion in self.explosion_list if explosion.tick()]
        self.good_list = [thing for thing in self.good_list if self.process_good_object(thing)]
        self.bad_list = [thing for thing in self.bad_list if self.process_bad_object(thing)]
        
        self.elapsed_ticks += 1
        self.level_number = int(math.log10(self.elapsed_ticks))
        
        self.random_new_things()
        
        
    def random_new_things(self):
        for i in range(int(self.level_number)):
            where = self.get_random_start_point()
            what = random.choice(self.random_things)
            how_fast = random.choice(self.random_velocities)
            do_mirror_sprite = random.choice((True, False))
            if what == "good":
                thing_sprite = sprite.AnimatedSprite(images.IMG_GOOD_THINGS, 1, 4)
                thing_sprite.reset_animation(random.choice(self.good_sprite_choices), do_mirror_sprite)
                self.good_list.append(FallingThing(thing_sprite, where, how_fast))
            elif what == "bad":
                thing_sprite = sprite.AnimatedSprite(images.IMG_BAD_THINGS, 3, 3)
                thing_sprite.reset_animation(random.choice(self.bad_sprite_choices), do_mirror_sprite)
                self.bad_list.append(FallingThing(thing_sprite, where, how_fast))
                
        
    
    def process_good_object(self, good_object):
        good_object.fall()
        if good_object.collides_with(self.target):
            self.target.good_hit()
            self.score += self.GOOD_HIT_INCREMENT
            return False
        elif good_object.has_reached(self.MAX_DEPTH):
            self.score -= self.GOOD_MISSED_DECREMENT
            return False
        else:
            return True
    
    def process_bad_object(self, bad_object):
        bad_object.fall()
        if bad_object.collides_with(self.target):
            self.target.bad_hit()
            self.explosion_list.append(Explosion(bad_object.draw_rect.center))
            return False
        elif bad_object.has_reached(self.MAX_DEPTH):
            self.explosion_list.append(Explosion(bad_object.draw_rect.center))
            return False
        else:
            return True
    
    def get_random_start_point(self):
        return (int(random.uniform(0,800)), -30)
    
    def draw(self, surface):
        for goodThing in self.good_list: goodThing.draw(surface)
        for badThing in self.bad_list: badThing.draw(surface)
        for explosion in self.explosion_list: explosion.draw(surface)
        

class Explosion(object):
    def __init__(self, center_point):
        self.draw_rect = pygame.Rect(0, 0, 50, 50)
        self.draw_rect.center = center_point
        self.counter = 12
        self.animation = sprite.AnimatedSprite(images.IMG_EXPLOSION, 12, 1)
        self.do_mirror = random.choice((True, False))
        
    def tick(self):
        self.animation.next_frame(0, self.do_mirror)
        self.counter -= 1
        return (self.counter > 0)
        
    def draw(self, dest_surface):
        self.animation.draw(dest_surface, self.draw_rect)
   
   
class FallingThing(object):
    def __init__(self, sprite, center_point, velocity):
        self.sprite = sprite
        self.hit_rect = pygame.Rect(0, 0, 20, 20)
        self.draw_rect = pygame.Rect(0, 0, 30, 30)
        self.hit_rect.center = center_point
        self.draw_rect.center = center_point
        self.velocity = velocity
        
    def fall(self):
        self.draw_rect.move_ip(0, self.velocity)
        self.hit_rect.move_ip(0, self.velocity)
        self.sprite.next_frame()
        
    def has_reached(self, y_max):
        return self.hit_rect.bottom >= y_max
        
    def draw(self, surface):
        self.sprite.draw(surface, self.draw_rect)
        # debug output
        # pygame.draw.rect(surface, 0x0000ff, self.hit_rect, 1)
        
    def collides_with(self, other):
        return self.hit_rect.colliderect(other.hit_rect)
        